package ludumdare18.behaviors
{
    import flash.geom.Point;
    import ludumdare18.G;
    import ludumdare18.gameobjects.Enemy;
    import stego.collections.LinkedListNode;
    import stego.GameObject;
    import stego.util.scene.GameObjectHolder;

	/**
     * ...
     * @author Mike Meyer
     */
    public class EnemySpawning
    {
        public var layer:GameObjectHolder;
        public var timer:Number = 0;
        public static const TOTAL_TIME:Number = 60;
        public function get timeleft():Number { return Math.max(0, TOTAL_TIME - timer); }

        protected var paused:Boolean = false;

        protected var spawnRules:Array = [ new SpawnRule(EnemyBehavior, 0, 5, 1.5),
                                           new SpawnRule(EnemyBehavior, 5, 10, 2),
                                           new SpawnRule(EnemyBehavior, 8.5, 11.5, 1),
                                           new SpawnRule(HoppingEnemyBehavior, 10.5, 40, 2),
                                           new SpawnRule(BigHopEnemyBehavior, 28, 60, 4),
                                           new SpawnRule(HoppingEnemyBehavior, 40, 60, 2) ];

        public function EnemySpawning(layer:GameObjectHolder)
        {
            this.layer = layer;
        }

        public function update(elapsedTime:Number):void
        {
            if (paused) { return; }

            timer += elapsedTime;

            for each (var spawnRule:SpawnRule in spawnRules)
            {
                if ( (spawnRule.minTime <= timer) && (timer <= spawnRule.maxTime))
                {
                    spawnRule.countdown -= elapsedTime;

                    if (spawnRule.countdown <= 0)
                    {
                        spawnRule.countdown += spawnRule.timeBetweenSpawns;

                        var rightside:Boolean = (Math.random() < 0.5);

                        // enemies
                        var enemy:Enemy = layer.add(Enemy) as Enemy;
                        enemy.x = (rightside)?G.gameBounds.right:G.gameBounds.left;
                        enemy.y = G.gameBounds.top - (0.5 * enemy.height);

                        enemy.behavior = new spawnRule.behaviorClass(enemy, G.game.inputUtil, rightside);
                        enemy.addBehavior(enemy.behavior);
                    }
                }
            }
        }

        public function resetClock():void
        {
            timer = 0;
        }

        public function clear():void
        {
            layer.forEach(clearIfEnemy);
        }

        protected function clearIfEnemy(object:GameObject):void
        {
            var enemy:Enemy = object as Enemy;
            if (enemy)
            {
                layer.remove(object);
            }
        }

        public function pause():void
        {
            paused = true;
        }

        public function unpause():void
        {
            paused = false;
        }
    }
}

class SpawnRule
{
    /** what to spawn */
    public var behaviorClass:Class;
    /** how late we must be in order to spawn it */
    public var minTime:Number;
    /** stop spawning this far in */
    public var maxTime:Number;
    /** time between spawns */
    public var timeBetweenSpawns:Number;
    /** countdown until the next one is spawned */
    public var countdown:Number;

    public function SpawnRule(behaviorClass:Class, minTime:Number, maxTime:Number, timeBetweenSpawns:Number)
    {
        this.behaviorClass = behaviorClass;
        this.minTime = minTime;
        this.maxTime = maxTime;
        this.timeBetweenSpawns = timeBetweenSpawns;
        countdown = 0;
    }
}
