package ludumdare18.behaviors
{
	import stego.behavior.Base;
    import stego.GameInterface;
    import stego.GameObject;
	
	/**
     * behavior for adding parallaxing to layers/objects.
     * @author Mike Meyer
     */
    public class ParallaxBehavior extends Base
    {
        public var game:GameInterface;
        public var offsetX:Number;
        public var offsetY:Number;

        /**
         * how much camera movement affects this.
         * You usually want your layers to go from 0 to 1 as they go further back.
         * 1 matches camera movement (good for super faraway stuff like the sun)
         * 0 is normal
         * negative values can be used to make it look like the camera is revolving around something
         * >1 is good for foreground stuff that should scroll by quickly
         */
        public var scrollScale:Number = 0.5;

        public function ParallaxBehavior( character:GameObject,
                                          game:GameInterface,
                                          scrollScale:Number,
                                          offsetX:Number,
                                          offsetY:Number )
        {
            super(character, null);
            this.game = game;
            this.scrollScale = scrollScale;
            this.offsetX = offsetX;
            this.offsetY = offsetY;
        }

        override public function update(elapsedTime:Number):void
        {
            super.update(elapsedTime);

            character.x = ((game.cameraX  - (0.5 * game.stageWidth)) * scrollScale) + offsetX;
            character.y = ((game.cameraY - (0.5 * game.stageHeight)) * scrollScale) + offsetY;
        }
    }

}
