package ludumdare18.gameobjects
{
    import flash.filters.BitmapFilterQuality;
    import flash.filters.BlurFilter;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import stego.GameInterface;
	import stego.gameobject.CanvasGameObject;
    import stego.util.ArrayUtil;
    import stego.util.Easing;
    import stego.util.MathUtil;
	
	/**
     * ...
     * @author Mike Meyer
     */
    public class Background extends CanvasGameObject
    {
        protected var _functions:Array;
        protected const POINTS_PER_FRAME:int = 1;
        protected const ITERATIONS:int = 512;
        protected const DRAW_AFTER:int = 20;

        protected static var tempPoint:Point = new Point();

        protected var countdown:Number = 0;


        public function get timeBetweenPatternChanges():Number { return _timeBetweenPatternChanges; }
        public function set timeBetweenPatternChanges(value:Number):void
        {
            _timeBetweenPatternChanges = value;
            if (countdown > _timeBetweenPatternChanges)
            {
                countdown = _timeBetweenPatternChanges;
            }
        }
        private var _timeBetweenPatternChanges:Number = 30.0;

        public var randomizeFunctionOrder:Boolean = false;
        public var scrollX:int = 0;
        public var scrollY:int = 0;

        /**
         * randomized coeffecients used in the flame functions below
         */
        protected var coeffs:Array;


        protected var coeffTweenStarts:Array;
        protected var coeffTweenEnds:Array;

        protected static const blurFilter:BlurFilter = new BlurFilter(1.1,1.1, BitmapFilterQuality.LOW);

        override public function init(game:GameInterface):void
        {
            super.init(game);

            _functions = [func3, func4, func6, func7];

            coeffs = [0, 0, 0, 0, 0, 0, 0];
            coeffTweenStarts = [1, 1, 1, 1, 1, 1, 1];
            coeffTweenEnds = [1, 1, 1, 1, 1, 1, 1];

            for (var i:int = 0; i < coeffs.length; ++i)
            {
                coeffs[i] = 0.1 - 0.2 * Math.random();
            }

            newPattern();
        }

        protected function newPattern():void
        {
            for (var i:int = 0; i < coeffs.length; ++i)
            {
                coeffTweenStarts[i] = coeffs[i];
                coeffTweenEnds[i] = 1.0 - 2.0 * Math.random();
            }

            countdown = _timeBetweenPatternChanges;
        }

        override public function update(elapsedTime:Number):void
        {
            super.update(elapsedTime);

            countdown -= elapsedTime;
            if (countdown <= 0)
            {
                newPattern();
            }
            var t:Number = Easing.overshootInOut(1 - (countdown / _timeBetweenPatternChanges));
            for (var i:int = 0; i < coeffs.length; ++i)
            {
                coeffs[i] = MathUtil.lerp(coeffTweenStarts[i], coeffTweenEnds[i], t);
            }

            _canvas.fade(0.0005);
            /*if (Math.random() < 0.05)
            {
                if (_canvas.target.bitmapData)
                {
                    tempPoint.x = 0;
                    tempPoint.y = 0;
                    _canvas.target.bitmapData.applyFilter(_canvas.target.bitmapData, _canvas.target.bitmapData.rect, tempPoint, blurFilter);
                }
            }*/

            tempPoint.x = 1.0 - 2.0 * Math.random();
            tempPoint.y = 1.0 - 2.0 * Math.random();

            var r:Number, g:Number, b:Number, hue:Number, color:Number;
            hue = (2.0 * Math.PI * coeffs[6]);
            r = 255 * 0.5 * (1.0 + Math.cos(hue) );
            g = 255 * 0.5 * (1.0 + Math.cos(hue + (2.0*Math.PI/3.0)) );
            b = 255 * 0.5 * (1.0 + Math.cos(hue + (4.0*Math.PI/3.0)) );
            color = 0xFF << 24 | uint(r) << 16 | uint(g) << 8 | uint(b);

            for (var pnt:int = 0; pnt < POINTS_PER_FRAME; ++pnt)
            {
                for (i = 0; i < ITERATIONS; ++i)
                {
                    if (randomizeFunctionOrder)
                    {
                        ArrayUtil.randomOneOf(_functions)(tempPoint);
                    }
                    else
                    {
                        _functions[i % _functions.length](tempPoint);
                    }

                    if (i >= DRAW_AFTER)
                    {
                        plot(tempPoint, color);
                    }
                }
            }
            _canvas.target.bitmapData.scroll(scrollX, scrollY);
        }

        /**
         * draw a dot at the given location
         */
        protected function plot(point:Point, color:Number):void
        {
            var x:Number = point.x;
            var y:Number = point.y;

            x = 0.5 + (0.5 * x);
            y = 0.5 + (0.5 * y);

            x *= width;
            y *= height;

            _canvas.setPixel(x,y,color);
        }

        /**
         * take two coordinates each in [-1,1] and changes it to another point in that range
         */
        public function func3(point:Point):void
        {
            var x:Number = point.x;
            var y:Number = point.y;
            var rSq:Number = (x * x) + (y * y);
            var sin:Number = Math.sin(rSq);
            var cos:Number = Math.cos(rSq);
            point.x = (x*sin) - (y*cos);
            point.y = (x*cos) + (y*sin);
        }

        /**
         * take two coordinates each in [-1,1] and changes it to another point in that range
         */
        public function func4(point:Point):void
        {
            var x:Number = point.x;
            var y:Number = point.y;
            point.x = x + (coeffs[0] * Math.sin(Math.tan( 3 * y )));
            point.y = y + (coeffs[1] * Math.sin(Math.tan( 3 * x )));
        }

        /**
         * take two coordinates each in [-1,1] and changes it to another point in that range
         */
        public function func6(point:Point):void
        {
            var x:Number = point.x;
            var y:Number = point.y;
            var r:Number = Math.sqrt((x * x) + (y * y));
            point.x = (x + y)*(x - y) / r;
            point.y = 2*x*y/r;
        }

        /**
         * take two coordinates each in [-1,1] and changes it to another point in that range
         */
        public function func7(point:Point):void
        {
            var x:Number = point.x;
            var y:Number = point.y;
            point.x = 0.75 * (coeffs[2] * x * Math.abs(x) + coeffs[3]);
            point.y = 0.75 * (coeffs[4] * y * Math.abs(y) + coeffs[5]);
        }
    }

}
