package ludumdare18.gameobjects
{
    import flash.display.Bitmap;
    import flash.display.Sprite;
    import ludumdare18.behaviors.EnemyBehavior;
    import ludumdare18.G;
    import ludumdare18.LD18Sounds;
    import stego.behavior.platformer.Jump;
    import stego.behavior.platformer.Run;
    import stego.GameInterface;
    import stego.constants.Colors;
    import stego.GameObject;
    import stego.util.graphics.AnimatedSpriteSheet;
    import stego.util.InputUtil;
    import stego.gameobject.PlatformerCharacter;
    import stego.util.collision.CollisionInfo;
    import stego.util.Sounds;

    public class Enemy extends PlatformerCharacter
    {
        [Embed(source='../../../assets/enemy.png')]
        protected static const _SPRITESHEET_CLASS:Class;
        protected static const SPRITESHEET_DATA:Object = { bitmapData:new _SPRITESHEET_CLASS().bitmapData,
                                                           frameWidth:48,
                                                           frameHeight:48 };
        /** util for rendering sprites out of a spritesheet */
        protected var _spriteSheet:AnimatedSpriteSheet;

        public var behavior:EnemyBehavior;

        public function get carried():Boolean { return behavior.carried; }
        public function set carried(value:Boolean):void { behavior.carried = value; }
        public function get thrown():Boolean { return behavior.thrown; }
        public function set thrown(value:Boolean):void { behavior.thrown = value; }

        public override function init(game:GameInterface):void
        {
            super.init(game);

            createGraphics();
            affectedByGravity = false;
            restart();
        }

        override public function restart():void
        {
            super.restart();
        }

        protected function createGraphics():void
        {
            if (!_spriteSheet)
            {
                var bitmap:Bitmap = new Bitmap(null);
                addChild(bitmap);
                _spriteSheet = new AnimatedSpriteSheet( bitmap,
                                                        SPRITESHEET_DATA.bitmapData,
                                                        SPRITESHEET_DATA.frameWidth,
                                                        SPRITESHEET_DATA.frameHeight );
                _spriteSheet.addAnimation("idle", [0,1], 1.0, true);
                _spriteSheet.addAnimation("carried", [2,3], 2.0, true);
                _spriteSheet.addAnimation("thrown", [4,5], 8.0, true);
                bitmap.x = -0.5 * bitmap.width;
                bitmap.y = -0.5 * bitmap.height;
            }
        }

        override public function update(elapsedTime:Number):void
        {
            super.update(elapsedTime);

            if (thrown)
            {
                if (_spriteSheet.currentAnim != "thrown")
                {
                    _spriteSheet.playAnimation("thrown");
                }
            }
            else if (carried)
            {
                if (_spriteSheet.currentAnim != "carried")
                {
                    _spriteSheet.playAnimation("carried");
                }
            }
            else
            {
                if (_spriteSheet.currentAnim != "idle")
                {
                    _spriteSheet.playAnimation("idle");
                }
            }

            if (dying && !G.gameBounds.intersects(getCollisionBounds()))
            {
                alive = false;
            }

            _spriteSheet.update(elapsedTime);
        }

        override public function collisionOccurred(ci:CollisionInfo, object:GameObject):void
        {
            if (dying) { return; }
            var enemy:Enemy = object as Enemy;
            if (enemy)
            {
                if (enemy.thrown)
                {
                    dying = true;
                    thrown = true;
                    enemy.dying = true;
                    G.game.cameraShake = 4;

                    Sounds.playSound(LD18Sounds.ENEMYDIE_SOUND);
                }
            }
            else
            {
                if (object == null)
                {
                    super.collisionOccurred(ci, object);
                    behavior.thrown = false;
                }
            }
        }
    }
}
