package ludumdare18.gameobjects
{
    import flash.display.Bitmap;
    import flash.display.Sprite;
    import flash.geom.Rectangle;
    import ludumdare18.behaviors.EnemyBehavior;
    import ludumdare18.G;
    import ludumdare18.LD18Sounds;
    import stego.behavior.platformer.Jump;
    import stego.behavior.platformer.Run;
    import stego.constants.Keyboard;
    import stego.GameInterface;
    import stego.constants.Colors;
    import stego.GameObject;
    import stego.util.graphics.AnimatedSpriteSheet;
    import stego.util.InputUtil;
    import stego.gameobject.PlatformerCharacter;
    import stego.util.collision.CollisionInfo;
    import stego.util.Sounds;

    public class Player extends PlatformerCharacter
    {
        [Embed(source='../../../assets/player.png')]
        protected static const _PLAYER_SPRITESHEET_CLASS:Class;
        protected static const PLAYER_SPRITESHEET_DATA:Object = { bitmapData:new _PLAYER_SPRITESHEET_CLASS().bitmapData,
                                                                  frameWidth:32,
                                                                  frameHeight:32 };
        /** util for rendering sprites out of a spritesheet */
        protected var _spriteSheet:AnimatedSpriteSheet;

        /** */
        protected var grabKeys:Array = [Keyboard.CONTROL, Keyboard.SPACE, Keyboard.X];

        /** behavior class for the running */
        protected var _runBehavior:Run;
        /** behvior class for the jumping */
        protected var _jumpBehavior:Jump;


        /** keep track of what enemy it is touching */
        protected var carriedEnemy:Enemy;

        /** keep track of what enemy it is touching */
        protected var touchingEnemy:Enemy;

        protected var throwSpeed:Number = 300;

        protected var hurt:Boolean = false;

        public override function init(game:GameInterface):void
        {
            super.init(game);

            createGraphics();
            createBehaviors(game.inputUtil);
            carriedEnemy = null;
            touchingEnemy = null;
            restart();
        }

        protected function createGraphics():void
        {
            if (!_spriteSheet)
            {
                var bitmap:Bitmap = new Bitmap(null);
                addChild(bitmap);
                _spriteSheet = new AnimatedSpriteSheet( bitmap,
                                                        PLAYER_SPRITESHEET_DATA.bitmapData,
                                                        PLAYER_SPRITESHEET_DATA.frameWidth,
                                                        PLAYER_SPRITESHEET_DATA.frameHeight );
                _spriteSheet.addAnimation("idle", [0], 1.0, true);
                _spriteSheet.addAnimation("jump", [1], 1.0, true);
                _spriteSheet.addAnimation("run", [1, 2, 1,0], 12.0, true);
                _spriteSheet.addAnimation("idle_carry", [3], 1.0, true);
                _spriteSheet.addAnimation("jump_carry", [4], 1.0, true);
                _spriteSheet.addAnimation("run_carry", [4,5,4,3], 12.0, true);
                _spriteSheet.addAnimation("hurt", [6,7], 16.0, true);
                bitmap.x = -0.5 * bitmap.width;
                bitmap.y = -0.5 * bitmap.height;
            }
        }

        /**
         * add behaviors to this object
         */
        protected function createBehaviors(inputUtil:InputUtil):void
        {
            if (!_runBehavior)
            {
                _runBehavior = new Run(this, inputUtil);
                _runBehavior.airAccel = 1000;
                _runBehavior.airDecel = 1000;
                _runBehavior.maxRunSpeed = 200;
                _runBehavior.runAccel = 1000;
                _runBehavior.stopAccel = 1000;
            }
            addBehavior(_runBehavior);
            if (!_jumpBehavior)
            {
                _jumpBehavior = new Jump(this, inputUtil);
                _jumpBehavior.jumpSpeed = 500;
                _jumpBehavior.jumpKeys = [Keyboard.SHIFT, Keyboard.UP, Keyboard.Z, Keyboard.C];
                _jumpBehavior.numAirJumps = 0;
                _jumpBehavior.jumpCallback = jumpCallback;
            }
            addBehavior(_jumpBehavior);
        }

        private function jumpCallback():void
        {
            Sounds.playSound(LD18Sounds.JUMP_SOUND);
        }

        override public function preCollision(timeDelta:Number):void
        {
            super.preCollision(timeDelta);
            touchingEnemy = null;
        }

        override public function collisionOccurred(ci:CollisionInfo, object:GameObject):void
        {
            // hitting something makes you unhurt
            hurt = false;

            var enemy:Enemy = object as Enemy;
            if ((null != enemy) && !enemy.carried && !enemy.thrown)
            {
                if (ci.normal.y < -0.7)
                {
                    // touching the top is ok
                    touchingEnemy = enemy;

                    var collisionBox:Rectangle = this.getCollisionBounds();
                    var offsetIntoCollisionBoxFromBottom:Number = y - collisionBox.bottom;
                    y = ci.location.y + offsetIntoCollisionBoxFromBottom;
                    if (dy > 0.0) { dy = 0.0; }
                    isOnGround = true;
                }
                else
                {
                    // touching anything else is not.
                    dropEnemy();
                    var kickSpeedX:Number = 600;
                    var kickSpeedY:Number = 200;
                    dx = (enemy.x<x) ? kickSpeedX : -kickSpeedX;
                    dy = -kickSpeedY;
                    G.game.cameraShake = 10;
                    G.lost = true;
                    hurt = true;

                    Sounds.playSound(LD18Sounds.DIE_SOUND);
                }
            }
            else if ((null == enemy) || (!enemy.carried && !enemy.thrown))
            {
                super.collisionOccurred(ci, object);
            }
        }

        public function dropEnemy():void
        {
            if (null != carriedEnemy)
            {
                carriedEnemy.carried = false;
                carriedEnemy = null;
            }
        }

        override public function update(elapsedTime:Number):void
        {
            if (_jumpBehavior.isJumping && touchingEnemy)
            {
                touchingEnemy = null;
            }

            if (touchingEnemy && (touchingEnemy != carriedEnemy))
            {
                dx -= touchingEnemy.dx;
            }

            super.update(elapsedTime);

            if (carriedEnemy)
            {
                carriedEnemy.x = x;
                carriedEnemy.y = y - 0.5 * (height + carriedEnemy.height);
                carriedEnemy.dx = 0;
                carriedEnemy.dy = 0;

                if (G.game.inputUtil.anyJustPressed(grabKeys))
                {
                    throwEnemy();
                    Sounds.playSound(LD18Sounds.THROW_SOUND);
                }
            }
            else
            {
                if (G.game.inputUtil.anyJustPressed(grabKeys))
                {
                    if (null != touchingEnemy)
                    {
                        carriedEnemy = touchingEnemy;
                        carriedEnemy.carried = true;
                        Sounds.playSound(LD18Sounds.PICKUP_SOUND);
                    }
                }
            }

            updateAnims(elapsedTime);

            if (touchingEnemy && (touchingEnemy != carriedEnemy))
            {
                dx += touchingEnemy.dx;
                if (!_jumpBehavior.isJumping)
                {
                    dy += 300;
                }
            }
        }

        override public function integrate(elapsedTime:Number):void
        {
            super.integrate(elapsedTime);
        }

        protected function updateAnims(elapsedTime:Number):void
        {
            var suffix:String = (null != carriedEnemy)?"_carry":"";
            if (hurt)
            {
                if (_spriteSheet.currentAnim != "hurt")
                {
                    _spriteSheet.playAnimation("hurt");
                }
            }
            else if (isOnGround)
            {
                var absDx:Number = Math.abs(dx);

                if (absDx > 0.5)
                {
                    if (_spriteSheet.currentAnim != "run"+suffix)
                    {
                        _spriteSheet.playAnimation("run"+suffix);
                    }
                }
                else
                {
                    if (_spriteSheet.currentAnim != "idle"+suffix)
                    {
                        _spriteSheet.playAnimation("idle"+suffix);
                    }
                }
            }
            else
            {
                if (_spriteSheet.currentAnim != "jump"+suffix)
                {
                    _spriteSheet.playAnimation("jump"+suffix);
                }
            }

            _spriteSheet.update(elapsedTime);

        }

        protected function throwEnemy():void
        {
            if (null != carriedEnemy)
            {
                carriedEnemy.carried = false;
                carriedEnemy.thrown = true;
                carriedEnemy.dx = (scaleX > 0) ? throwSpeed : -throwSpeed;
                carriedEnemy.dy = -throwSpeed;
                carriedEnemy = null;
            }
        }
    }
}
