package ludumdare18.gamestate
{
    import flash.geom.Rectangle;
    import ludumdare18.behaviors.EnemySpawning;
    import ludumdare18.behaviors.ParallaxBehavior;
    import ludumdare18.G;
    import ludumdare18.gameobjects.Clock;
    import ludumdare18.gameobjects.Player;
    import stego.constants.Colors;
    import stego.GameInterface;
    import stego.gameobject.PlatformerCharacter;
    import stego.GameState;
    import stego.ui.Fader;
    import stego.util.collision.CollisionInfo;
    import stego.util.MathUtil;
    import stego.util.ObjectPool;
    import stego.util.scene.GameObjectHolder;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Playing extends GameState
    {
        protected var _player:Player;
        protected var _characterLayer:GameObjectHolder;
        protected var _guiLayer:GameObjectHolder;
        protected var _fader:Fader;

        /**
         * create scene
         */
        public override function begin(game:GameInterface):void
        {
            super.begin(game);

            game.physics.gravity = 1000;
            game.physics.maxFallSpeed = 400;

            _characterLayer = game.layers.addLayer();
            G.spawner = new EnemySpawning(_characterLayer);

            game.physics.addLayer(_characterLayer);
            game.physics.addCollisionPair(_characterLayer, _characterLayer);

            var w:Number = game.stageWidth;
            var h:Number = game.stageHeight;

            // player
            _player = _characterLayer.add(Player) as Player;
            _player.x = G.gameBounds.x + (0.50 * G.gameBounds.width);
            _player.y = G.gameBounds.y + (0.50 * (G.gameBounds.height - _player.height));

            _guiLayer = game.layers.addLayer();
            _guiLayer.add(Clock);
            _fader = _guiLayer.add(Fader) as Fader;
            _fader.addBehavior(new ParallaxBehavior(_fader, game, 1, 0.5 * game.stageWidth, 0.5 * game.stageHeight));

            game.cameraSpeed = 450;
            game.cameraFollowTarget = _player;
            game.cameraFollowRect = G.gameBounds.clone();
            game.cameraFollowRect.x += (0.5 * w);
            game.cameraFollowRect.width -= w;
            game.cameraFollowRect.y += (0.5 * h);
            game.cameraFollowRect.height -= h;

            game.cheats.addCheatCode("skip", skipAhead);
            game.cheats.addCheatCode("lose", lose);
            game.cheats.addCheatCode("win", win);

            startGame();
        }

        override public function end():void
        {
            super.end();

            game.cheats.removeCheatCode("skip", skipAhead);
            game.cheats.removeCheatCode("lose", lose);
            game.cheats.removeCheatCode("win", win);
            game.layers.remove(_characterLayer);
            game.layers.remove(_guiLayer);
            game.cameraFollowTarget = null;
            game.cameraX = 0.5 * game.stageWidth;
            game.cameraY = 0.5 * game.stageHeight;
        }

        protected function startGame():void
        {
            G.spawner.clear();
            G.spawner.pause();
            G.spawner.resetClock();
            _fader.fadeToColor(0, Colors.BLACK);
            _fader.fadeBack(1.0, fadeInComplete);
        }
        protected function fadeInComplete():void
        {
            G.spawner.unpause();
            G.lost = false;
        }
        protected function lose():void
        {
            G.spawner.pause();
            _fader.fadeToColor(1, Colors.BLACK, startGame);
        }
        protected function win():void
        {
            G.spawner.clear();
            _fader.fadeToColor(0, Colors.WHITE, winFadeComplete);
        }

        protected function winFadeComplete():void
        {
            game.changeGameState(new Victory());
        }

        protected function skipAhead():void
        {
            G.spawner.timer = 30;
        }

        override public function collision():void
        {
            super.collision();

            _characterLayer.forEach(checkEdgeCollision);
        }

        override public function update():void
        {
            super.update();

            game.cameraShake = MathUtil.moveToward(game.cameraShake, 0, 10 * game.time.timeDelta);
            G.spawner.update(game.time.timeDelta);

            if (G.lost && !_fader.fadeInProgress)
            {
                lose();
            }

            if (!G.lost && (G.spawner.timeleft==0))
            {
                win();
            }
        }

        public function checkEdgeCollision(go:PlatformerCharacter):void
        {
            //early out, dying characters can just fall off
            if (go.dying) { return; }

            var ci:CollisionInfo = ObjectPool.getObject(CollisionInfo);
            var characterBounds:Rectangle = ObjectPool.getObject(Rectangle);
            characterBounds = go.getCollisionBounds(characterBounds);
            if (characterBounds.left < G.gameBounds.left)
            {
                ci.t = 0;
                ci.normal.x = 1;
                ci.normal.y = 0;
                ci.location.x = G.gameBounds.left;
                ci.location.y = 0.5 * (characterBounds.top + characterBounds.bottom);

                // let the character know he hit something
                go.collisionOccurred(ci, null);

                // update values and continue checking
                characterBounds = go.getCollisionBounds( characterBounds );
            }
            if (characterBounds.right > G.gameBounds.right)
            {
                ci.t = 0;
                ci.normal.x = -1;
                ci.normal.y = 0;
                ci.location.x = G.gameBounds.right;
                ci.location.y = 0.5 * (characterBounds.top + characterBounds.bottom);

                // let the character know he hit something
                go.collisionOccurred(ci, null);

                // update values and continue checking
                characterBounds = go.getCollisionBounds( characterBounds );
            }
            /*if (characterBounds.top < G.gameBounds.top)
            {
                ci.t = 0;
                ci.normal.x = 0;
                ci.normal.y = 1;
                ci.location.x = 0.5 * (characterBounds.left + characterBounds.right);
                ci.location.y = G.gameBounds.top;

                // let the character know he hit something
                go.collisionOccurred(ci, null);

                // update values and continue checking
                characterBounds = go.getCollisionBounds( characterBounds );
            }*/
            if (characterBounds.bottom > G.gameBounds.bottom)
            {
                ci.t = 0;
                ci.normal.x = 0;
                ci.normal.y = -1;
                ci.location.x = 0.5 * (characterBounds.left + characterBounds.right);
                ci.location.y = G.gameBounds.bottom;

                // let the character know he hit something
                go.collisionOccurred(ci, null);

                // update values and continue checking
                characterBounds = go.getCollisionBounds( characterBounds );
            }
            ObjectPool.returnObject(Rectangle, characterBounds);
            ObjectPool.returnObject(CollisionInfo, ci);
        }
    }

}
