﻿package stego.behavior.platformer
{
    import stego.behavior.Base;
    import stego.behavior.IBehavior;
    import stego.constants.Keyboard;
    import stego.GameObject;
    import stego.gameobject.PlatformerCharacter;
    import stego.util.InputUtil;
    import stego.util.MathUtil;

    /**
     * behavior for a player that can run left and right
     * @author Mike Meyer
     */
    public class Run extends Base
    {
        // direction options
        protected static const LEFT:Number = -1;
        protected static const STOP:Number = 0;
        protected static const RIGHT:Number = 1;

        public var moveRightKey:Number = Keyboard.RIGHT;
        public var moveLeftKey:Number = Keyboard.LEFT;


        /** How quickly you slow down when no keys are pressed */
        public var stopAccel:Number = 500;

        /** How quickly you speed up when walking */
        public var runAccel:Number = 500;

        /** How quickly you speed up in the air */
        public var airAccel:Number = 187.5;

        /** How quickly you slow down in the air */
        public var airDecel:Number = 187.5;

        /** max running speed */
        public var maxRunSpeed:Number = 125;

        /** If he gets going faster than maxRunSpeed, this is how much acceleration we apply to fix it. */
        public var speedCappingAccel:Number = 500;

        /**
         * constructor
         * @param	character
         * @param	inputUtil
         */
        public function Run(character:GameObject, inputUtil:InputUtil)
        {
            super(character, inputUtil);
        }

        /**
         * process one frame of controls
         */
        override public function update(elapsedTime:Number):void
        {
            walkingControls(elapsedTime);
        }

        /**
         * controls for moving left and right
         */
        protected function walkingControls(elapsedTime:Number):void
        {
            var desiredDirection:Number = STOP;
            if (inputUtil.isDown(moveRightKey))
            {
                desiredDirection = RIGHT;
            }
            else if (inputUtil.isDown(moveLeftKey))
            {
                desiredDirection = LEFT;
            }

            doRunAccel(desiredDirection, elapsedTime);
        }

        /**
         * Handle the math to do the right kinds of acceleration
         * for platform character movement
         */
        protected function doRunAccel(desiredDirection:Number, elapsedTime:Number):void
        {
            var currentDirection:Number = 0;
            if (character.dx > 0)
            {
                currentDirection = RIGHT;
            }
            else if (character.dx < 0)
            {
                currentDirection = LEFT;
            }

            // controls a bit differently on the ground
            if (character.isOnGround)
            {
                // changing direction or stopping
                if ( (currentDirection != 0) && (desiredDirection != currentDirection) )
                {
                    character.dx = MathUtil.moveToward(character.dx, 0, stopAccel * elapsedTime);
                }
                else // accelerating
                {
                    character.dx = MathUtil.moveToward(character.dx, desiredDirection * maxRunSpeed, runAccel * elapsedTime);
                }
            }
            else
            {
                if (STOP == desiredDirection)
                {
                    character.dx = MathUtil.moveToward(character.dx, 0, airDecel * elapsedTime);
                }
                else
                {
                    character.dx = MathUtil.moveToward(character.dx, desiredDirection * maxRunSpeed, airAccel * elapsedTime);
                }
            }

            // if it gets over the max speed slow it down
            if (character.dx > maxRunSpeed)
            {
                character.dx = MathUtil.moveToward( character.dx, maxRunSpeed, speedCappingAccel * elapsedTime);
            }
            if (character.dx < -maxRunSpeed)
            {
                character.dx = MathUtil.moveToward( character.dx, -maxRunSpeed, speedCappingAccel * elapsedTime);
            }

            // flip graphics
            var platformerCharacter:PlatformerCharacter = character as PlatformerCharacter;
            if (platformerCharacter)
            {
                if (desiredDirection > 0)
                {
                    platformerCharacter.faceRight();
                }
                if (desiredDirection < 0)
                {
                    platformerCharacter.faceLeft();
                }
            }
        }
    }

}
