﻿package stego.debug
{
    import flash.display.DisplayObjectContainer;
    import flash.display.Sprite;
    import flash.events.MouseEvent;
    import flash.utils.getQualifiedClassName;
    import stego.GameInterface;
    import stego.GameObject;
    import stego.util.scene.GameObjectHolder;
    import stego.collections.LinkedList;
    import stego.constants.Colors;
    import stego.util.ObjectPool;
    import stego.util.StringUtil;
    import stego.ui.TextButton;
    import stego.ui.TextField;

    /**
     * debugging tools
     * @author Mike Meyer
     */
    public class DebugWidget extends GameObject
    {
        protected var _logButton:TextButton;
        protected var _poolStatusButton:TextButton;
        protected var _reusablesButton:TextButton;
        protected var _profilingButton:TextButton;

        protected var _log:TextField;
        protected var _poolStatus:TextField;
        protected var _reusablesStatus:TextField;
        protected var _profilingStatus:TextField;

        protected static const BACKGROUND_ALPHA:Number = 0.6;

        /** pixels of extra space between the buttons */
        protected static const BUTTON_SPACING:Number = 20;

        /** pixels betwee the left side of the screen and where debug text starts */
        protected static const LEFT_MARGIN:Number = 10;

        /** store the game in case debugging gets enabled later */
        protected var game:GameInterface;

        /**
         *
         */
        override public function init(game:GameInterface):void
        {
            super.init(game);

            this.game = game;

            if (0 == numChildren) // hack to see if it has been init'ed
            {
                createBackground(game.stageWidth, game.stageHeight);
                createTabBar();
                if (!_log) { _log = createStatusTextField(); }
                if (!_poolStatus) { _poolStatus = createStatusTextField(); }
                if (!_reusablesStatus) { _reusablesStatus = createStatusTextField(); }
                if (!_profilingStatus) { _profilingStatus = createStatusTextField(); }

                internalGameObjects.add(FpsCounter);
            }

            // default to log view
            goToLog();

            // starts out invisible
            this.visible = false;
        }

        /**
         * make the background to go behind everything
         */
        protected function createBackground( width:Number, height:Number ):void
        {
            graphics.beginFill(0x000000, BACKGROUND_ALPHA);
            graphics.drawRect(
                0,
                0,
                width,
                height );
            graphics.endFill();
        }

        /**
         * create the clickable list of tabs
         */
        protected function createTabBar():void
        {
            if (!_logButton)
            {
                _logButton = internalGameObjects.add(TextButton) as TextButton;
                _logButton.setText("log");
                _logButton.x = LEFT_MARGIN;
                _logButton.y = 20;
                _logButton.setFontColor(Colors.WHITE);
                _logButton.setHoverColor(Colors.YELLOW);
                _logButton.addEventListener(MouseEvent.CLICK, goToLog);

                _poolStatusButton = internalGameObjects.add(TextButton) as TextButton;
                _poolStatusButton.setText("pool");
                _poolStatusButton.x = _logButton.x + _logButton.width + BUTTON_SPACING;
                _poolStatusButton.y = 20;
                _poolStatusButton.setFontColor(Colors.WHITE);
                _poolStatusButton.setHoverColor(Colors.YELLOW);
                _poolStatusButton.addEventListener(MouseEvent.CLICK, goToPoolStatus);

                _reusablesButton = internalGameObjects.add(TextButton) as TextButton;
                _reusablesButton.setText("go reuse");
                _reusablesButton.x = _poolStatusButton.x + _poolStatusButton.width + BUTTON_SPACING;
                _reusablesButton.y = 20;
                _reusablesButton.setFontColor(Colors.WHITE);
                _reusablesButton.setHoverColor(Colors.YELLOW);
                _reusablesButton.addEventListener(MouseEvent.CLICK, goToReusablesStatus);

                _profilingButton = internalGameObjects.add(TextButton) as TextButton;
                _profilingButton.setText("profiling");
                _profilingButton.x = _reusablesButton.x + _reusablesButton.width + BUTTON_SPACING;
                _profilingButton.y = 20;
                _profilingButton.setFontColor(Colors.WHITE);
                _profilingButton.setHoverColor(Colors.YELLOW);
                _profilingButton.addEventListener(MouseEvent.CLICK, goToProfiling);
            }
        }

        /**
         * react to press of the log button
         */
        private function goToLog(e:MouseEvent= null):void
        {
            hideAllTabs();
            _log.visible = true;
        }

        /**
         * react to press of the pool button
         */
        private function goToPoolStatus(e:MouseEvent= null):void
        {
            hideAllTabs();
            _poolStatus.visible = true;
        }

        /**
         * react to press of the reusables button
         */
        private function goToReusablesStatus(e:MouseEvent= null):void
        {
            hideAllTabs();
            _reusablesStatus.visible = true;
        }

        /**
         * react to press of the profiling button
         */
        private function goToProfiling(e:MouseEvent= null):void
        {
            hideAllTabs();
            _profilingStatus.visible = true;
        }

        /**
         * hide all the objects that are in these half-assed tabs
         */
        protected function hideAllTabs():void
        {
            _log.visible = false;
            _poolStatus.visible = false;
            _reusablesStatus.visible = false;
            _profilingStatus.visible = false;
        }

        /**
         * make the log display
         */
        protected function createLog():void
        {
            if (!_log)
            {
                _log = internalGameObjects.add(TextField) as TextField;
                _log.y = 40;
            }
        }

        /**
         * make a status display widget
         */
        protected function createStatusTextField():TextField
        {
            var tf:TextField = internalGameObjects.add(TextField) as TextField;
            tf.setFontColor(Colors.WHITE);
            tf.x = LEFT_MARGIN;
            tf.y = 60;
            return tf;
        }

        /**
         * add to the log
         */
        public function log(text:String):void
        {
            if (_log)
            {
                // TODO:  make into a ginormous scrolling thing someday
                _log.setText(text);
            }
        }

        /**
         *
         */
        override public function update(elapsedTime:Number):void
        {
            super.update(elapsedTime);
            var text:String, c:Object, key:String, ll:LinkedList;

            if (visible)
            {
                if (_poolStatus && _poolStatus.visible)
                {
                    text = "object pool info\n'new's:  " + (ObjectPool.numNews + ObjectPool.numBitmapDataNews + ObjectPool.numLinkedListNodeNews );
                    for (c in ObjectPool.pools)
                    {
                        ll = ObjectPool.pools[c];
                        text += "\n" + getQualifiedClassName(c) + ":  " + ll.count;
                    }
                    for (key in ObjectPool.bitmapDataPool)
                    {
                        ll = ObjectPool.bitmapDataPool[key];
                        text += "\nbitmapData " + key + ":  " + ll.count;
                    }
                    text += "\nlinkedListNode:  " + ObjectPool.linkedListNodePool.length;

                    _poolStatus.setText(text);
                }
                if (_reusablesStatus && _reusablesStatus.visible)
                {
                    text = "reusable game objects info\n'new's:  " + GameObjectHolder.numNews;
                    for (c in GameObjectHolder.reusables)
                    {
                        var a:Array = GameObjectHolder.reusables[c];
                        text += "\n" + getQualifiedClassName(c) + ":  " + a.length;
                    }
                    _reusablesStatus.setText(text);
                }
                if (_profilingStatus && _profilingStatus.visible)
                {
                    text = "Profiling info\n";
                    text += StringUtil.fixedWidth("name", 25);
                    text += StringUtil.fixedWidth("times", 8);
                    text += StringUtil.fixedWidth("avg", 6);
                    text += StringUtil.fixedWidth("min", 4);
                    text += StringUtil.fixedWidth("max", 4);
                    text += StringUtil.fixedWidth("total", 6);
                    for (var sectionName:String in Profiler.sections)
                    {
                        var ps:ProfilerSection = Profiler.sections[sectionName];
                        text += "\n" + StringUtil.fixedWidth(sectionName, 25);
                        text += StringUtil.fixedWidth(ps.times.toString(), 8);
                        text += StringUtil.fixedWidth(ps.avg.toFixed(3), 6);
                        text += StringUtil.fixedWidth(ps.min.toString(), 4);
                        text += StringUtil.fixedWidth(ps.max.toString(), 4);
                        text += StringUtil.fixedWidth(ps.total.toString(), 6);
                    }
                    _profilingStatus.setText(text);
                }
            }
        }

        /**
         * cleanup
         */
        override public function shutdown():void
        {
            super.shutdown();
            game = null;
        }
    }

}
