﻿package stego.ui
{
    import flash.geom.ColorTransform;
    import stego.constants.Colors;
    import stego.GameInterface;
    import stego.GameObject;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Fader extends GameObject
    {
        /**
         * A default, huge size so that we don't have to worry about whether it will cover everything
         */
        private static const FADER_SIZE:Number = 1000;

        private var _timeRemaining:Number = 0.0;
        private var _fadeTime:Number = 0.0;
        private var _fadeIn:Boolean = true;
        private var _callback:Function;

        private var _graphicsDrawn:Boolean = false;

        public function get fadeInProgress():Boolean { return (_timeRemaining > 0); }

        public override function init(game:GameInterface):void
        {
            super.init(game);
            createGraphics();

            // start invisible
            visible = false;
            alpha = 0;
        }

        protected function createGraphics():void
        {
            if (!_graphicsDrawn)
            {
                _graphicsDrawn = true;
                graphics.beginFill(Colors.WHITE);
                // making it extra big to avoid camera shake artifacts.
                // for the future, some things should be unscrollable/unshakeable
                graphics.drawRect(
                    -0.5 * FADER_SIZE,
                    -0.5 * FADER_SIZE,
                    FADER_SIZE,
                    FADER_SIZE );
                graphics.endFill();
            }
        }

        public override function update(elapsedTime:Number):void
        {
            super.update(elapsedTime);
            _timeRemaining -= elapsedTime;

            if (_timeRemaining > 0.0)
            {
                var t:Number = (_timeRemaining / _fadeTime);
                alpha = _fadeIn ? 1-t : t;
            }
            else
            {
                alpha = _fadeIn ? 1.0 : 0.0;

                if (!_fadeIn)
                {
                    visible = false;
                }

                if (null != _callback)
                {
                    var temp:Function = _callback;
                    _callback = null;
                    temp();
                }
            }
        }

        /**
         * fade to a solid color.
         */
        public function fadeToColor(duration:Number, color:uint, callback:Function=null):void
        {
            var colorTransform:ColorTransform = this.transform.colorTransform;
            colorTransform.redMultiplier   = ( (color & 0xFF0000) >> 16) / 255;
            colorTransform.greenMultiplier = ( (color & 0x00FF00) >> 8) / 255;
            colorTransform.blueMultiplier  = ( (color & 0x0000FF) ) / 255;
            this.transform.colorTransform  = colorTransform;

            visible = true;
            alpha = (duration > 0) ? 0 : 1;
            _timeRemaining = duration;
            _fadeTime = duration;
            _fadeIn = true;
            _callback = callback;
        }

        /**
         * fade back after FadeToColor
         */
        public function fadeBack(duration:Number, callback:Function=null):void
        {
            alpha = 1;
            visible = true;
            _timeRemaining = duration;
            _fadeTime = duration;
            _fadeIn = false;
            _callback = callback;
        }
    }

}
