﻿package stego.ui
{
    import flash.display.DisplayObject;
    import flash.display.Sprite;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import stego.GameInterface;
    import stego.GameObject;

    /**
     * ...
     * @author Mike Meyer
     */
    public class GraphicButton extends GameObject
    {
        protected var _graphicClass:Class;
        protected var _graphic:DisplayObject;

        private var mouseEventDummy:Sprite;

        /**
         * Graphic to display for the button when the mouse is not over it
         */
        public function set graphic(value:Class):void
        {
            _graphicClass = value;
            if (!mouseIsOver)
            {
                swapGraphic(_graphicClass);
            }
        }
        public function get graphic():Class { return _graphicClass; }

        /**
         * Graphic to display for the button when the mouse is over it
         */
        public function get hoverGraphic():Class { return _hoverGraphic; }
        public function set hoverGraphic(value:Class):void
        {
            _hoverGraphic = value;

            if (mouseIsOver)
            {
                swapGraphic(_hoverGraphic);
            }
        }
        protected var _hoverGraphic:Class;

        /**
         * whether the mouse is currently over the button
         */
        public function get mouseIsOver():Boolean { return _mouseIsOver; }
        public function set mouseIsOver(value:Boolean):void
        {
            _mouseIsOver = value;
            if (value && (null != hoverGraphic) )
            {
                swapGraphic(hoverGraphic);
            }
            if (!value && (null != graphic))
            {
                swapGraphic(graphic);
            }
        }
        protected var _mouseIsOver:Boolean = false;

        /**
         * initial setup/preparation for reuse
         */
        public override function init(game:GameInterface):void
        {
            super.init(game);

            if (null == mouseEventDummy)
            {
                mouseEventDummy = new Sprite();
                mouseEventDummy.graphics.beginFill(0, 0);
                mouseEventDummy.graphics.drawRect(0, 0, 1, 1);
                mouseEventDummy.graphics.endFill();
                this.addChild(mouseEventDummy);
                addMouseEventListeners();
            }

            mouseIsOver = false;
        }

        /**
         * cleanup
         */
        public override function shutdown():void
        {
            removeMouseEventListeners();
            super.shutdown();
        }

        /**
         * change the current graphic to graphicClass
         */
        protected function swapGraphic(graphicClass:Class):void
        {
            if (null != _graphic)
            {
                removeChild(_graphic);
            }
            _graphic = new _graphicClass();
            addChild(_graphic);

            removeChild(mouseEventDummy);
            addChild(mouseEventDummy);
            updateMouseEventDummy();
        }

        /**
         * make the dummy mouse event rect match the main graphic
         */
        private function updateMouseEventDummy():void
        {
            var bounds:Rectangle = _graphic.getBounds(mouseEventDummy.parent);
            mouseEventDummy.x = bounds.x;
            mouseEventDummy.y = bounds.y;
            mouseEventDummy.width = bounds.width;
            mouseEventDummy.height = bounds.height;
        }

        private function handleRollOver(evt:MouseEvent = null):void
        {
            mouseIsOver = true;
        }

        private function handleRollOut(evt:MouseEvent = null):void
        {
            mouseIsOver = false;
        }

        private function addMouseEventListeners():void
        {
            mouseEventDummy.addEventListener(MouseEvent.ROLL_OVER, handleRollOver);
            mouseEventDummy.addEventListener(MouseEvent.ROLL_OUT, handleRollOut);
        }

        private function removeMouseEventListeners():void
        {
            mouseEventDummy.removeEventListener(MouseEvent.ROLL_OVER, handleRollOver);
            mouseEventDummy.removeEventListener(MouseEvent.ROLL_OUT, handleRollOut);
        }
    }

}
