package stego.ui
{
    import flash.display.Sprite;
    import flash.events.MouseEvent;
    import flash.net.navigateToURL;
    import flash.net.URLRequest;
    import stego.constants.Colors;
    import stego.GameInterface;
    import stego.GameObject;
    import stego.script.Step;
    import stego.script.step.Animate;
    import stego.script.step.Loop;
    import stego.script.step.WaitTime;
    import stego.util.music.MusicEvent;
    import stego.util.music.MusicUtil;

    /**
     * Widget for displaying music credits
     * MAKE SURE YOU SET .musicUtil!
     * @author Mike Meyer
     */
    public class MusicCredits extends GameObject
    {
        private static const MUSIC_CREDITS_HEIGHT:Number = 45;
        private static const MUSIC_CREDITS_MARGIN:Number = 5;
        private static const MUSIC_CREDITS_INDENT:Number = 80;
        private static const MUSIC_CREDITS_ROW_HEIGHT:Number = 11;
        private static const LINK_COLOR:Number = 0x30C0FF;
        private static const BACKGROUND_ALPHA:Number = 0.75;

        private static const FADE_IN_TIME:Number = 1.0;
        private static const FULL_ALPHA_TIME:Number = 10.0;
        private static const FADE_OUT_TIME:Number = 2.0;

        /**
         * music utility to listen to
         */
        public function get musicUtil():MusicUtil { return _musicUtil; }
        public function set musicUtil(value:MusicUtil):void
        {
            if (null != _musicUtil)
            {
                _musicUtil.removeEventListener(MusicEvent.EVENT_NEW_SONG, songChanged);
            }
            _musicUtil = value;
            if (null != _musicUtil)
            {
                _musicUtil.addEventListener(MusicEvent.EVENT_NEW_SONG, songChanged);
            }
        }
        private var _musicUtil:MusicUtil;

        private var background:Sprite;
        private var musicLabel:TextField;
        private var musicTitle:TextField;
        private var musicArtist:TextField;
        private var musicLink:TextField;
        private var songLink:String;

        private var runScript:Boolean = false;
        private var animScript:Step;

        /**
         * this function called for both initial setup and to re-init for reuse
         */
        public override function init(game:GameInterface):void
        {
            super.init(game);

            createGraphics(game);

            animScript = new Loop(1, [
                new stego.script.step.Animate(this, { alpha:0 }, { alpha:1 }, FADE_IN_TIME),
                new WaitTime(FULL_ALPHA_TIME),
                new stego.script.step.Animate(this, {alpha:1}, {alpha:0}, FADE_OUT_TIME) ] );

            // start out invisible
            this.visible = false;
        }

        /**
         * clean up
         */
        public override function shutdown():void
        {
            musicUtil = null;
            super.shutdown();
        }

        /**
         * create the graphics for this widget
         */
        protected function createGraphics(game:GameInterface):void
        {
            if (null == background)
            {
                background = new Sprite();
                background.graphics.beginFill( Colors.BLACK_ENOUGH );
                background.graphics.drawRect(
                    0,
                    0,
                    game.stageWidth,
                    MUSIC_CREDITS_HEIGHT );
                background.graphics.endFill();
                this.addChild(background);
                background.mouseEnabled = false; // let mouseevents go through it

                background.alpha = BACKGROUND_ALPHA;
            }

            if (null == musicLabel)
            {
                musicLabel = new TextField();
                musicLabel.init(game);
                musicLabel.x = MUSIC_CREDITS_MARGIN;
                musicLabel.y = MUSIC_CREDITS_MARGIN;
                musicLabel.setFontSize(14);
                musicLabel.setText("Music");
                this.addChild(musicLabel);
                musicLabel.mouseEnabled = false; // let mouseevents go through it
            }

            if (null == musicTitle)
            {
                musicTitle = new TextField();
                musicTitle.init(game);
                musicTitle.x = MUSIC_CREDITS_MARGIN + MUSIC_CREDITS_INDENT;
                musicTitle.y = MUSIC_CREDITS_HEIGHT - ( (3*MUSIC_CREDITS_ROW_HEIGHT) + MUSIC_CREDITS_MARGIN);
                musicTitle.setFontSize(12);
                musicTitle.setText("");
                this.addChild(musicTitle);
                musicTitle.mouseEnabled = false; // let mouseevents go through it
            }

            if (null == musicArtist)
            {
                musicArtist = new TextField();
                musicArtist.init(game);
                musicArtist.x = MUSIC_CREDITS_MARGIN + MUSIC_CREDITS_INDENT;
                musicArtist.y = MUSIC_CREDITS_HEIGHT - ( (2*MUSIC_CREDITS_ROW_HEIGHT) + MUSIC_CREDITS_MARGIN);
                musicArtist.setFontSize(12);
                musicArtist.setText("");
                this.addChild(musicArtist);
                musicArtist.mouseEnabled = false; // let mouseevents go through it
            }

            if (null == musicLink)
            {
                musicLink = new TextField();
                musicLink.init(game);
                musicLink.x = MUSIC_CREDITS_MARGIN + MUSIC_CREDITS_INDENT;
                musicLink.y = MUSIC_CREDITS_HEIGHT - ( (1*MUSIC_CREDITS_ROW_HEIGHT) + MUSIC_CREDITS_MARGIN);
                musicLink.setFontSize(12);
                musicLink.setText("");
                this.addChild(musicLink);

                musicLink.setFontColor( LINK_COLOR );
                musicLink.buttonMode = true;
                musicLink.mouseChildren = false;
                musicLink.addEventListener(MouseEvent.CLICK, songLinkClicked);
            }
        }

        /**
         *
         */
        public override function update(elapsedTime:Number):void
        {
            super.update(elapsedTime);

            if (runScript)
            {
                var done:Boolean = animScript.execute(elapsedTime);

                if (done)
                {
                    runScript = false;
                    this.visible = false;
                }
            }
        }

        /**
         *
         */
        private function songChanged(e:MusicEvent):void
        {
            if (null == e.song.title)
            {
                musicTitle.visible = false;
            }
            else
            {
                musicTitle.setText( e.song.title );
                musicTitle.visible = true;
            }

            if (null == e.song.artist)
            {
                musicArtist.visible = false;
            }
            else
            {
                musicArtist.setText( "by " + e.song.artist );
                musicArtist.visible = true;
            }

            songLink = e.song.link;
            if (null == e.song.link)
            {
                musicLink.visible = false;
            }
            else
            {
                musicLink.setText( "[link]" );
                musicLink.visible = true;
            }

            if (null != e.song.title)
            {
                runScript = true;
                this.visible = true;
                animScript.restart();
            }
        }

        /**
         * open up the song's page in a new window
         */
        private function songLinkClicked(e:MouseEvent):void
        {
            navigateToURL(new URLRequest(songLink));
        }
    }
}
