package stego.ui
{
    import flash.display.Sprite;
    import flash.events.Event;
    import stego.constants.Colors;
    import stego.GameInterface;
    import stego.GameObject;
    import stego.ui.TextField;

    /**
     * widget for the pause overlay
     */
    public class PauseScreen extends GameObject
    {
        /** color for the background */
        public function get screenColor():uint { return _screenColor; }
        public function set screenColor(value:uint):void
        {
            if (_screenColor != value)
            {
                _screenColor = value;
                drawScrim();
            }
        }
        protected var _screenColor:uint;

        /** pause screen width. should probably be kept in sync with the game stage width */
        override public function get width():Number { return _width; }
        override public function set width(value:Number):void
        {
            if (value != _width)
            {
                _width = value;
                drawScrim();
                updatePositions();
            }
        }
        protected var _width:Number;

        /** pause screen height. should probably be kept in sync with the game stage height */
        override public function get height():Number { return _height; }
        override public function set height(value:Number):void
        {
            if (value != _height)
            {
                _height = value;
                drawScrim();
                updatePositions();
            }
        }
        protected var _height:Number;

        /** big label to put on the screen when paused */
        public function get title():String { return _title; }
        public function set title(value:String):void
        {
            if (_title != value)
            {
                _title = value;
                if (_titleTextField)
                {
                    _titleTextField.setText(value);
                    updatePositions();
                }
            }
        }
        protected var _title:String = "PAUSED";

        /** smaller text to go below the title */
        public function get instructions():String { return _instructions; }
        public function set instructions(value:String):void
        {
            if (_instructions != value)
            {
                instructions = value;
                if (_instructionsTextField)
                {
                    _instructionsTextField.setText(value);
                    updatePositions();
                }
            }
        }
        protected var _instructions:String = "click to continue";

        /** font to use on the pause screen */
        public function get font():String { return _font; }
        public function set font(value:String):void
        {
            if (_font != value)
            {
                _font = value;
                if (_titleTextField)
                {
                    _titleTextField.setFont(value);
                }
                if (_instructionsTextField)
                {
                    _instructionsTextField.setFont(value);
                }
                if (_titleTextField || _instructionsTextField)
                {
                    updatePositions();
                }
            }
        }
        protected var _font:String = TextField.defaultFont;

        /** color to draw the text with */
        public function get fontColor():uint { return _fontColor; }
        public function set fontColor(value:uint):void
        {
            if (_fontColor != value)
            {
                _fontColor = value;
                if (_titleTextField)
                {
                    _titleTextField.setFontColor(value);
                }
                if (_instructionsTextField)
                {
                    _instructionsTextField.setFontColor(value);
                }
            }
        }
        protected var _fontColor:uint = TextField.defaultFontColor;

        /** font size for the screen title */
        public function get titleFontSize():Number { return _titleFontSize; }
        public function set titleFontSize(value:Number):void
        {
            if (_titleFontSize != value)
            {
                _titleFontSize = value;
                if (_titleTextField)
                {
                    _titleTextField.setFontSize(value);
                }
            }
        }
        protected var _titleFontSize:Number = 4.0 * TextField.defaultFontSize;

        /** font size for the instructions text */
        public function get instructionsFontSize():Number { return _instructionsFontSize; }
        public function set instructionsFontSize(value:Number):void
        {
            if (_instructionsFontSize != value)
            {
                _instructionsFontSize = value;
                if (_instructionsTextField)
                {
                    _instructionsTextField.setFontSize(value);
                }
            }
        }
        protected var _instructionsFontSize:Number = 2.0 * TextField.defaultFontSize;

        /** alpha of the background scrim*/
        public function get scrimAlpha():Number { return _scrimAlpha; }
        public function set scrimAlpha(value:Number):void
        {
            _scrimAlpha = value;
            drawScrim();
        }
        protected var _scrimAlpha:Number = 0.75;

        /** color of the background scrim*/
        public function get scrimColor():uint { return _scrimColor; }
        public function set scrimColor(value:uint):void
        {
            _scrimColor = value;
            drawScrim();
        }
        protected var _scrimColor:uint = Colors.BLACK_ENOUGH;


        /** textfield for the big "PAUSED" */
        protected var _titleTextField:TextField

        /** textfield for the big "click to continue" */
        protected var _instructionsTextField:TextField

        /**
         * get the pause screen ready for display
         */
        override public function init(game:GameInterface):void
        {
            super.init(game);

            _width = game.stageWidth;
            _height = game.stageHeight;

            if (!_titleTextField)
            {
                _titleTextField = internalGameObjects.add(TextField) as TextField;
                _titleTextField.setText(_title);
                _titleTextField.setFont(_font);
                _titleTextField.setFontSize(_titleFontSize);
                _titleTextField.setFontColor(_fontColor);
            }

            if (!_instructionsTextField)
            {
                _instructionsTextField = internalGameObjects.add(TextField) as TextField;
                _instructionsTextField.setText(_instructions);
                _instructionsTextField.setFont(_font);
                _instructionsTextField.setFontSize(_instructionsFontSize);
                _instructionsTextField.setFontColor(_fontColor);
            }
            drawScrim();
            updatePositions();
        }

        /**
         * reposition the textfields
         */
        private function updatePositions():void
        {
            _titleTextField.centerOn( 0.5 * _width, 0.4 * _height );
            _instructionsTextField.centerOn( 0.5 * _width, 0.7 *_height );
        }

        /**
         * draw the background/background color
         */
        private function drawScrim():void
        {
            graphics.clear();
            graphics.beginFill(_scrimColor, _scrimAlpha);
            graphics.drawRect( 0, 0, _width, _height )
            graphics.endFill();
        }
    }
}
