﻿package stego.ui
{
    import flash.display.DisplayObject;
    import flash.display.DisplayObjectContainer;
    import flash.display.Sprite;
    import flash.geom.Rectangle;
    import stego.GameInterface;
    import stego.GameObject;
    import stego.util.MathUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PulsingGraphic extends GameObject
    {
        protected var mainGraphic:DisplayObject;
        protected var pulseGraphic:DisplayObject;
        protected var pulseHolder:DisplayObjectContainer;

        /**
         * Graphic to display for the button
         */
        public function get graphic():Class { return _buttonGraphic; }
        public function set graphic(value:Class):void
        {
            _buttonGraphic = value;
            swapGraphic(_buttonGraphic);
        }

        /**
         * change the current graphic to graphicClass
         */
        protected function swapGraphic(graphicClass:Class):void
        {
            if (null == pulseHolder)
            {
                pulseHolder = new Sprite();
                addChild(pulseHolder);
            }

            if (null != mainGraphic)
            {
                removeChild(mainGraphic);
            }
            mainGraphic = new graphicClass();
            addChild(mainGraphic);

            if (null != pulseGraphic)
            {
                pulseHolder.removeChild(pulseGraphic);
            }
            pulseGraphic = new graphicClass();
            pulseHolder.addChild(pulseGraphic);

            updatePulsePosition();
        }

        protected var _buttonGraphic:Class;

        public var pulseMaxScale:Number = 1.5;
        public var pulseMaxAlpha:Number = 0.5;
        public var pulseMinAlpha:Number = 0.0;
        public var pulseDuration:Number = 0.75;

        /** get/set the time between pulses */
        public function get pulsePeriod():Number { return _pulsePeriod; }
        public function set pulsePeriod(value:Number):void
        {
            if (value != _pulsePeriod)
            {
                _pulsePeriod = value;
            }
        }
        protected var _pulsePeriod:Number = 1.0;


        private var pulseTimer:Number = 0;

        public override function init(game:GameInterface):void
        {
            super.init(game);
        }

        public override function update(elapsedTime:Number):void
        {
            super.update(elapsedTime);

            pulseTimer += elapsedTime;
            if (pulseTimer > _pulsePeriod)
            {
                pulseTimer = 0;
            }

            var t:Number = Math.min(1.0, pulseTimer / pulseDuration);
            pulseHolder.alpha = MathUtil.lerp(pulseMaxAlpha, pulseMinAlpha, t);
            pulseHolder.scaleX = MathUtil.lerp( 1.0, pulseMaxScale, t);
            pulseHolder.scaleY = pulseHolder.scaleX;
        }

        /**
         * keep the pulse centered behind the main graphic
         */
        private function updatePulsePosition():void
        {
            var bounds:Rectangle = mainGraphic.getBounds(this);
            pulseHolder.x = 0.5 * (bounds.left + bounds.right);
            pulseHolder.y = 0.5 * (bounds.top + bounds.bottom);
            pulseGraphic.x = -0.5 * (bounds.left + bounds.right);
            pulseGraphic.y = -0.5 * (bounds.top + bounds.bottom);
        }

        /**
         * cleanup
         */
        override public function shutdown():void
        {
            super.shutdown();
            pulseHolder.scaleX = 1.0;
            pulseHolder.scaleY = 1.0;
            pulseTimer = 0;
        }
    }

}
