﻿package stego.ui
{
    import flash.display.Sprite;
    import flash.display.DisplayObjectContainer;
    import flash.geom.Rectangle;
    import stego.constants.Colors;
    import stego.GameInterface;
    import stego.GameObject;
    import stego.util.MathUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PulsingText extends GameObject
    {
        protected var pulseTextField:TextField;
        protected var textField:TextField;

        public function get hAlign():uint { return textField.hAlign; }
        public function set hAlign(value:uint):void
        {
            textField.hAlign = value;
            updatePulsePosition();
        }
        public function get vAlign():uint { return textField.vAlign; }
        public function set vAlign(value:uint):void
        {
            textField.vAlign = value;
            updatePulsePosition();
        }

        protected var color:Number = Colors.BLACK;

        private static const PULSE_MAX_SCALE:Number = 1.5;
        private static const PULSE_MAX_ALPHA:Number = 1.0;
        private static const PULSE_MIN_ALPHA:Number = 0.0;
        private static const PULSE_DURATION:Number = 0.75;

        // the pulse text seems to be too high, and I don't
        // have time to look into why
        private static const Y_FUDGE:Number = 0;// 0.0625;

        private var _pulseTimer:Number;

        /// how much time passes between pulses
        protected var pulseDelay:Number = 1.0;

        public function PulsingText()
        {
            super();

            resetDefaults();
        }

        public override function init(game:GameInterface):void
        {
            super.init(game);

            createGraphics();
            setFontColor(color);
        }

        public function setFontColor(color:Number):void
        {
            this.color = color;
            pulseTextField.setFontColor(color);
            textField.setFontColor(color);
        }

        protected function createGraphics():void
        {
            var holder:Sprite = new Sprite();
            if (null == textField)
            {
                textField = new TextField();
                holder.addChild(textField);
            }

            if (null == pulseTextField)
            {
                pulseTextField = new TextField();
                holder.addChild(pulseTextField);
            }

            this.addChild(holder);
        }

        public override function update(elapsedTime:Number):void
        {
            super.update(elapsedTime);

            _pulseTimer = (_pulseTimer + elapsedTime) % pulseDelay;

            var t:Number = Math.min(1.0, _pulseTimer / PULSE_DURATION);
            pulseTextField.alpha = MathUtil.lerp(PULSE_MAX_ALPHA, PULSE_MIN_ALPHA, t);
            pulseTextField.scaleX = MathUtil.lerp( 1.0, PULSE_MAX_SCALE, t);
            pulseTextField.scaleY = pulseTextField.scaleX;
        }

        /**
         * center the textfield on some point.  CALL SET TEXT *FIRST*
         */
        public function centerOn(x:Number, y:Number):void
        {
            this.x = x;
            this.y = y;
            textField.centerOn(0, 0);
            updatePulsePosition();
        }

        /**
         * Change the displayed text
         */
        public function setText(text:String):void
        {
            textField.setText(text);

            pulseTextField.setText(text);
            updatePulsePosition();
        }

        /**
         * Change the displayed font size.
         */
        public function setFontSize(desiredSize:Number):void
        {
            textField.setFontSize(desiredSize);

            pulseTextField.setFontSize(desiredSize);
            updatePulsePosition();
        }

        /**
         * reset to defaults.  This thing will likely get re-used!
         */
        public override function shutdown():void
        {
            resetDefaults();

            pulseTextField.alpha = PULSE_MAX_ALPHA;
            pulseTextField.scaleX = 1.0;
            pulseTextField.scaleY = 1.0;

            super.shutdown();
        }

        /**
         * reset to defaults.  This thing will likely get re-used!
         */
        private function resetDefaults():void
        {
            _pulseTimer = PULSE_DURATION;

            if (null != pulseTextField)
            {
                pulseTextField.resetDefaults();
            }

            if (null != textField)
            {
                textField.resetDefaults();
            }
        }

        /**
         * Change the font.  WILL SCREW UP ALIGNMENT.
         */
        public function setFont(font:String):void
        {
            textField.setFont(font);

            pulseTextField.setFont(font);
            updatePulsePosition();
        }

        /**
         * Update the pulse TextField's position to keep it centered on the real text
         */
        public function updatePulsePosition():void
        {
            var bounds:Rectangle = textField.getBounds(textField.parent);
            pulseTextField.centerOn(
                bounds.left + 0.5*bounds.width,
                bounds.top + 0.5*bounds.height + (Y_FUDGE * bounds.height ) );
        }
    }

}
