﻿package stego.util
{
    import adobe.utils.CustomActions;
    import flash.display.DisplayObject;
    import flash.display.DisplayObjectContainer;
    import flash.display.Sprite;
    import flash.geom.ColorTransform;
    import flash.geom.Point;
    import stego.GameObject;
    import stego.util.scene.GameObjectHolder;
    /**
     * utility functions for building up game graphics
     * @author Mike Meyer
     */
    public class GraphicsUtil
    {
        /// create a new game object in the layer, draw a rect on it.
        /// returns the new GameObject
        public static function addRectGameObject(layer:GameObjectHolder, x:Number, y:Number, width:Number, height:Number, color:Number, centerRect:Boolean = false):GameObject
        {
            var go:GameObject = layer.add();
            go.graphics.beginFill(color);
            if (centerRect)
            {
                go.graphics.drawRect(
                    -0.5 * width,
                    -0.5 * height,
                    width,
                    height );
            }
            else
            {
                go.graphics.drawRect(
                    0,
                    0,
                    width,
                    height );
            }
            go.graphics.endFill();
            go.x = x;
            go.y = y;
            return go;
        }
        /// create a new game object in the layer, draw a rect on it.
        /// returns the new GameObject
        public static function addRoundedBoxGameObject( layer:GameObjectHolder,
                                                        x:Number,
                                                        y:Number,
                                                        width:Number,
                                                        height:Number,
                                                        color:Number,
                                                        cornerRadius:Number,
                                                        centerRect:Boolean = false ):GameObject
        {
            var go:GameObject = layer.add();
            go.graphics.beginFill(color);
            if (centerRect)
            {
                go.graphics.drawRoundRect(
                    -0.5 * width,
                    -0.5 * height,
                    width,
                    height,
                    2 * cornerRadius,
                    2 * cornerRadius );
            }
            else
            {
                go.graphics.drawRoundRect(
                    0,
                    0,
                    width,
                    height,
                    2 * cornerRadius,
                    2 * cornerRadius );
            }
            go.graphics.endFill();
            go.x = x;
            go.y = y;
            return go;
        }

        /// add a graphic to the display object conrtainer and center it
        /// returns a reference to the added DisplayObject
        public static function addImageCentered(holder:DisplayObjectContainer, imageClass:Class):DisplayObject
        {
            return addImageCenteredOn(holder, imageClass, 0, 0);
        }

        /// add a graphic to the display object conrtainer and center it
        /// returns a reference to the added DisplayObject
        public static function addImageCenteredOn(holder:DisplayObjectContainer, imageClass:Class, x:Number, y:Number):DisplayObject
        {
            var dispObj:DisplayObject = new imageClass();
            dispObj.x = x + (-0.5 * dispObj.width);
            dispObj.y = y + (-0.5 * dispObj.height);
            holder.addChild(dispObj);
            return dispObj;
        }

        /**
         * remap from one display object's coordinate system to another
         */
        public static function remapCoordinate(coord:Point, from:DisplayObject, to:DisplayObject):Point
        {
            var temp:Point = from.localToGlobal(coord);
            return to.globalToLocal(temp);
        }

        /**
         * convert between a color hex code to a colortransform
         */
        public static function colorTransformFromHex(hex:Number):ColorTransform
        {
            var r:Number = ((hex >> 16) & 0xFF) / 255.0;
            var g:Number = ((hex >> 8) & 0xFF) / 255.0;
            var b:Number = ((hex) & 0xFF) / 255.0;
            return new ColorTransform( r, g, b );
        }

        /**
         * take a displayobject and stretch it between two points
         * ASSUMES the object's registration point is in its center
         */
        public static function stretchBetween( object:DisplayObject,
                                               pointA:Point,
                                               pointB:Point,
                                               overshootBefore:Number = 0.0,
                                               overshootAfter:Number = 0.0,
                                               maxLen:Number = 0.0 ):void
        {
            var angle:Number = Math.atan2(pointB.y - pointA.y, pointB.x - pointA.x);

            var overshotPointA:Point = new Point( pointA.x - overshootBefore * Math.cos(angle),
                                                  pointA.y - overshootBefore * Math.sin(angle) );
            var overshotPointB:Point = new Point( pointB.x + overshootAfter * Math.cos(angle),
                                                  pointB.y + overshootAfter * Math.sin(angle) );

            object.rotation = 0;
            var distance:Number = Point.distance(overshotPointA, overshotPointB);
            if (distance == 0.0)
            {
                distance = 1.0;
            }

            var scale:Number = 1.0;

            if ( (0.0 != maxLen) && (distance > maxLen) )
            {
                scale = maxLen / distance;
            }

            object.width = distance * scale;

            var center:Point = Point.interpolate(overshotPointB, overshotPointA, 0.5 * scale);
            object.x = center.x;
            object.y = center.y;

            object.rotation = angle * 180.0 / Math.PI;
        }

        /**
         * removes any children and graphics from teh sprite, so it should be as good as new
         */
        public static function clearSprite(s:Sprite ):void
        {
            s.graphics.clear();
            while (s.numChildren > 0) { s.removeChildAt(0); }
        }
    }

}
