package stego.util
{
    import flash.display.DisplayObject;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import flash.ui.ContextMenuBuiltInItems;
    import stego.GameObject;
    import stego.util.scene.GameObjectHolder;
    import stego.ui.InputText;
    import stego.ui.GraphicButton;
    import stego.ui.PulsingGraphicButton;
    import stego.ui.PulsingTextButton;
    import stego.ui.TextButton;
    import stego.ui.TextField;

    /**
     * Utility for putting simple guis together easily
     * @author Mike Meyer
     */
    public class GuiBuilder
    {
        public var font:String;
        public var fontColor:Number;
        public var highlightColor:Number;
        public var fontSize:Number;
        public var layer:GameObjectHolder;

        private var _area:Rectangle;

        function GuiBuilder(usableArea:Rectangle, layer:GameObjectHolder)
        {
            font = TextField.defaultFont;
            fontColor = TextField.defaultFontColor;
            fontSize = TextField.defaultFontSize;
            highlightColor = TextButton.defaultHighlightColor;
            this.layer = layer;
            _area = usableArea;
        }

        /**
         * helper for the add text functions
         * handles everything but the positioning
         */
        private function addTextHelper(text:String):TextField
        {
            var tf:TextField = layer.add(TextField) as TextField;
            tf.setText(text);
            tf.setFont(font);
            tf.setFontColor(fontColor);
            tf.setFontSize(fontSize);
            return tf;
        }

        /**
         * add an uncentered button to the layer
         * @param	text     text to display in it
         * @param	callback function to get called when the button is clicked
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function addButton(text:String, callback:Function, x_pct:Number, y_pct:Number):TextButton
        {
            var b:TextButton = layer.add(TextButton) as TextButton;
            b.setText(text);
            b.setFont(font);
            b.setFontColor(fontColor);
            b.setFontSize(fontSize);
            b.setHoverColor(highlightColor);
            b.addEventListener(MouseEvent.CLICK, callback);
            b.x = _area.left + (_area.width * x_pct);
            b.y = _area.top + (_area.height * y_pct);
            return b;
        }

        /**
         * Add a button to the layer
         * @param	text    text to display in it
         * @param	callback function to get called when the button is clicked
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function addButtonCentered(text:String, callback:Function, x_pct:Number, y_pct:Number):TextButton
        {
            var b:TextButton = addButton(text, callback, 0, 0);
            b.centerOn( _area.left + (_area.width * x_pct),
                        _area.top  + (_area.height * y_pct) );
            return b;
        }

        /**
         * add an uncentered button to the layer
         * @param	text     text to display in it
         * @param	callback function to get called when the button is clicked
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function addPulsingButton(text:String, callback:Function, x_pct:Number, y_pct:Number):PulsingTextButton
        {
            var b:PulsingTextButton = layer.add(PulsingTextButton) as PulsingTextButton;
            b.setText(text);
            b.setFont(font);
            b.setFontColor(fontColor);
            b.setFontSize(fontSize);
            b.setHoverColor(highlightColor);
            b.addEventListener(MouseEvent.CLICK, callback);
            b.x = _area.left + (_area.width * x_pct);
            b.y = _area.top + (_area.height * y_pct);
            return b;
        }

        /**
         * Add a button to the layer
         * @param	text    text to display in it
         * @param	callback function to get called when the button is clicked
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function addPulsingButtonCentered(text:String, callback:Function, x_pct:Number, y_pct:Number):PulsingTextButton
        {
            var b:PulsingTextButton = addPulsingButton(text, callback, 0, 0);
            b.centerOn( _area.left + (_area.width * x_pct),
                        _area.top  + (_area.height * y_pct) );
            return b;
        }



        /**
         * add an uncentered button to the layer
         * @param	text     text to display in it
         * @param	callback function to get called when the button is clicked
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function addPulsingGraphicButton(graphic:Class, hoverGraphic:Class, callback:Function, x_pct:Number, y_pct:Number):PulsingGraphicButton
        {
            var b:PulsingGraphicButton = layer.add(PulsingGraphicButton) as PulsingGraphicButton;
            b.graphic = graphic;
            b.hoverGraphic = hoverGraphic;
            b.addEventListener(MouseEvent.CLICK, callback);
            b.x = _area.left + (_area.width * x_pct);
            b.y = _area.top + (_area.height * y_pct);
            return b;
        }


        /**
         * add a centered button to the layer
         * @param	text     text to display in it
         * @param	callback function to get called when the button is clicked
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function addPulsingGraphicButtonCentered(graphic:Class, hoverGraphic:Class, callback:Function, x_pct:Number, y_pct:Number):PulsingGraphicButton
        {
            var b:PulsingGraphicButton = layer.add(PulsingGraphicButton) as PulsingGraphicButton;
            b.graphic = graphic;
            b.hoverGraphic = hoverGraphic;
            b.addEventListener(MouseEvent.CLICK, callback);
            b.x = _area.left + (_area.width * x_pct) - (0.5 * b.width);
            b.y = _area.top + (_area.height * y_pct) - (0.5 * b.height);
            return b;
        }
        /**
         * add a centered button to the layer
         * @param	graphic  bitmap class to use in the created GraphicButton
         * @param	callback function to get called when the button is clicked
         * @param	x_pct    how far across to put it,
         *                   0 = all the way to the left
         *                   1.0 = all the way to the right,
         * @param	y_pct    how far down to put it,
         *                   0 = at the top
         *                   1.0 = at the bottom
         * TODO: noncentered version.  If I even want one.
         */
        public function addGraphicButtonCentered(graphic:Class, hoverGraphic:Class, callback:Function, x_pct:Number, y_pct:Number):GraphicButton
        {
            var b:GraphicButton = layer.add(GraphicButton) as GraphicButton;
            b.graphic = graphic;
            b.hoverGraphic = hoverGraphic;
            b.addEventListener(MouseEvent.CLICK, callback);
            b.x = _area.left + (_area.width * x_pct) - (0.5 * b.width);
            b.y = _area.top + (_area.height * y_pct) - (0.5 * b.height);
            return b;
        }
        /**
         * add an image to the layer centered on a point
         * @param	graphic  bitmap class to use in the created GraphicButton
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         * TODO: noncentered version.  If I even want one.
         */
        public function addGraphicCentered(graphic:Class, x_pct:Number, y_pct:Number):GameObject
        {
            var go:GameObject = layer.add(GameObject);
            go.addChild(new graphic());
            go.x = _area.left + (_area.width * x_pct) - (0.5 * go.width);
            go.y = _area.top + (_area.height * y_pct) - (0.5 * go.height);
            return go;
        }

        /**
         * Add a textfield to the layer
         * @param	text    text to display in it
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function addText(text:String, x_pct:Number, y_pct:Number):TextField
        {
            var tf:TextField = addTextHelper(text);
            tf.x = _area.left + (_area.width * x_pct);
            tf.y = _area.top + (_area.height * y_pct);
            return tf;
        }

        /**
         * Add a textfield to the layer
         * @param	text    text to display in it
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function addTextCentered(text:String, x_pct:Number, y_pct:Number):TextField
        {
            var tf:TextField = addTextHelper(text);
            tf.centerOn( _area.left + (_area.width * x_pct),
                         _area.top + (_area.height * y_pct) );
            return tf;
        }

        /**
         * Add a scrim to darken anything in back of it and make foreground elements stand out
         * @param	opacity
         * @param	color
         * @return
         */
        public function addScrim(opacity:Number, color:Number = 0x000000):GameObject
        {
            var go:GameObject = layer.add();
            go.graphics.beginFill(color);
            go.graphics.drawRect(
                _area.left,
                _area.top,
                _area.width,
                _area.height);
            go.graphics.endFill();
            go.alpha = opacity;
            return go;
        }

        /**
         * Add an inputtext widget to the layer
         * @param	text    text to display in it
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function addInputText(text:String, x_pct:Number, y_pct:Number):InputText
        {
            var it:InputText = layer.add(InputText) as InputText;
            it.text = text;
            it.font = font;
            it.fontColor = fontColor;
            it.fontSize = fontSize;
            it.x = _area.left + (_area.width * x_pct);
            it.y = _area.top + (_area.height * y_pct);
            return it;
        }
    }
}
