package stego.util
{
    import flash.display.BitmapData;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import flash.utils.Dictionary;
    import stego.collections.LinkedList;
    import stego.collections.LinkedListNode;
	/**
     * A place to get and cache objects from to avoid new'ing them where possible
     * @author Mike Meyer
     */
    public class ObjectPool
    {
        public static var tempPoint:Point = new Point();


        /** object pools keyed by class */
        public static var pools:Dictionary = new Dictionary();
        /**
         * get an instance of the cached class
         * expect it to be in an invalid state and clean it up yourself
         */
        public static function getObject(c:Class):*
        {
            if (c in pools)
            {
                var ll:LinkedList = pools[c];
                if (ll.count)
                {
                    return ll.pop_back();
                }
            }

            ++numNews;
            return new c();
        }
        /**
         * return an instance to the cache
         * it is a good idea to remove any references
         * it has to other objects when you return it!
         */
        public static function returnObject(c:Class, instance:*):void
        {
            var ll:LinkedList;
            if (c in pools)
            {
                ll = pools[c];
            }
            else
            {
                pools[c] = ll = new LinkedList();
            }

            ll.push_back(instance);
        }

        /** stats on object pool: how many times it has done a 'new' */
        public static var numNews:uint = 0;


        /** BitmapData needs a special pool because you can't resize them after construction */
        public static var bitmapDataPool:Object = {};
        /**
         * get a bitmapData. expect it to be dirty!
         */
        public static function getBitmapData(width:int, height:int):BitmapData
        {
            var key:String = "" + width + "," + height;
            if (bitmapDataPool.hasOwnProperty(key))
            {
                var ll:LinkedList = bitmapDataPool[key];
                if (ll.count)
                {
                    return ll.pop_back();
                }
            }

            ++numBitmapDataNews;
            return new BitmapData(width, height);
        }
        /**
         * return an instance to the cache
         */
        public static function returnBitmapData(instance:BitmapData):void
        {
            var ll:LinkedList;
            var key:String = "" + instance.width + "," + instance.height;
            if (bitmapDataPool.hasOwnProperty(key))
            {
                ll = bitmapDataPool[key];
            }
            else
            {
                bitmapDataPool[key] = ll = new LinkedList();
            }

            ll.push_back(instance);
        }
        /** how many times it has done a 'new' for bitmapDatas */
        public static var numBitmapDataNews:uint = 0;

        /**
         * LinkedListNodes need a special pool because otherwise they try to
         * store themselves and things get wacky
         */
        public static var linkedListNodePool:Array = [];
        /**
         * get a LinkedListNode. expect it to be dirty!
         */
        public static function getLinkedListNode():LinkedListNode
        {
            var node:LinkedListNode = null;
            var numCachedNodes:uint = linkedListNodePool.length;
            if (numCachedNodes > 0)
            {
                node = linkedListNodePool[numCachedNodes - 1];
                linkedListNodePool.length = numCachedNodes - 1;
            }
            else
            {
                ++numLinkedListNodeNews;
                node = new LinkedListNode();
            }
            return node;
        }
        /**
         * return an instance to the cache
         */
        public static function returnLinkedListNode(instance:LinkedListNode):void
        {
            instance.clear();
            linkedListNodePool[linkedListNodePool.length] = instance;
        }
        /** how many times it has done a 'new' for these */
        public static var numLinkedListNodeNews:uint = 0;
    }
}
