package stego.util
{
    import adobe.utils.CustomActions;
    import stego.util.scene.GameObjectHolder;
    import stego.script.Step;
    import stego.script.step.Condition;
    import stego.script.step.CreateGameObjectFromBitmap;
    import stego.script.step.Loop;
    import stego.script.step.RunFunc;
    import stego.script.step.SpawnGameObject;
    import stego.script.step.Wait;

    /**
     * Utility for putting together scripts easily
     * @author Mike Meyer
     */
    public class ScriptBuilder
    {
        private var _script:/*stego.script.Step*/Array;

        /**
         * get rid of any script still in progress and start a new one
         */
        public function beginScript():void
        {
            _script = [];
        }

        /**
         * finish the current script
         */
        public function endScript():Step
        {
            var temp:/*stego.script.Step*/Array = _script;
            _script = null;
            return new Loop(1, temp);
        }

        /**
         * Add a step to block until a condition is met
         * @param	f
         */
        public function addCondition(f:Function):void
        {
            if (null != _script)
            {
                _script.push(new Condition(f));
            }
        }

        /**
         * Add a GameObject spawn to the script
         * @param	layer        layer the object should appear on
         * @param	bitmapClass  image to create it from
         * @param	centerX      x coord to center it on
         * @param	centerY      y coord to center it on
         */
        public function addSpawnFromBitmap(layer:GameObjectHolder, bitmapClass:Class, centerX:Number, centerY:Number):void
        {
            if (null != _script)
            {
                _script.push(new CreateGameObjectFromBitmap(layer, bitmapClass, centerX, centerY));
            }
        }

        /**
         * Add a function call to the script
         * @param	f  function to call
         */
        public function addFunc(f:Function):void
        {
            if (null != _script)
            {
                _script.push(new RunFunc(f));
            }
        }

        /**
         * Add an GameObject spawn to the script
         * @param	layer      layer the object should appear on
         * @param	enemyType  GameObject class
         * @param	howMany    how many copies you want
         */
        public function addObjectSpawn(layer:GameObjectHolder, enemyType:Class, howMany:Number = 1):void
        {
            if (null != _script)
            {
                _script.push(new SpawnGameObject(layer, enemyType, howMany));
            }
        }

        /**
         * Add a wait to the script
         * @param	frames how many frames to wait
         */
        public function addWait(frames:Number = 1):void
        {
            if (null != _script)
            {
                _script.push(new Wait(frames));
            }
        }
    }
}
