package stego.util.graphics
{
    import flash.display.Bitmap;
    import flash.display.BitmapData;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import stego.GameInterface;
    import stego.GameObject;

	/**
     * util for rendering sprites out of a spritesheet
     * @author Mike Meyer
     */
    public class SpriteSheet
    {
        /** image data from which to get the images for each tile. */
        public function get spriteSheetImage():BitmapData { return spriteSheetImage; }
        public function set spriteSheetImage(value:BitmapData):void
        {
            _spriteSheetImage = value;
            _spriteSheetColumns = _spriteSheetImage.width / _spriteWidth;
            _spriteSheetRows = _spriteSheetImage.height / _spriteHeight;
            _numFrames = _spriteSheetColumns * _spriteSheetRows;
            updateCanvas();
        }
        protected var _spriteSheetImage:BitmapData;

        /** width of each sprite. */
        public function get spriteWidth():uint { return _canvas.width; }
        public function set spriteWidth(value:uint):void
        {
            _canvas.width = value;
            _spriteSheetColumns = _spriteSheetImage.width / _spriteWidth;
            _numFrames = _spriteSheetColumns * _spriteSheetRows;
            updateCanvas();
        }
        protected var _spriteWidth:uint;

        /** height of each sprite. */
        public function get spriteHeight():uint { return _canvas.height; }
        public function set spriteHeight(value:uint):void
        {
            _canvas.height = value;
            _spriteSheetRows = _spriteSheetImage.height / _spriteHeight;
            _numFrames = _spriteSheetColumns * _spriteSheetRows;
            updateCanvas();
        }
        protected var _spriteHeight:uint;

        /** set which frame to show */
        public function get currentFrame():uint { return _currentFrame; }
        public function set currentFrame(value:uint):void
        {
            if (_currentFrame != value)
            {
                _currentFrame = value % _numFrames;
                updateCanvas();
            }
        }
        protected var _currentFrame:uint = 0;

        /** how many columns of frames are in the sprite sheet */
        protected var _spriteSheetColumns:uint = 0;
        /** how many rows of frames are in the sprite sheet */
        protected var _spriteSheetRows:uint = 0;
        /** how many frames are in the source image*/
        protected var _numFrames:uint = 0;

        /** Bitmap object it draws on */
        public function get target():Bitmap { return _canvas.target; }
        public function set target(value:Bitmap):void { _canvas.target = value; }

        /** canvas utility used to render the sprite */
        protected var _canvas:Canvas;

        /** so that we don't have the overhead of new'ing all the time */
        protected static var _tempRect:Rectangle = new Rectangle();
        /** so that we don't have the overhead of new'ing all the time */
        protected static const ORIGIN:Point = new Point(0,0);

        /**
         * constructor
         */
        public function SpriteSheet(target:Bitmap = null, source:BitmapData = null, spriteWidth:int = 100, spriteHeight:int = 100):void
        {
            _canvas = new Canvas(target, spriteWidth, spriteHeight);

            // let the setters get things up to date
            _spriteSheetImage = source;
            _spriteWidth = spriteWidth;
            _spriteHeight = spriteHeight;

            if (_spriteSheetImage)
            {
                _spriteSheetColumns = _spriteSheetImage.width / _spriteWidth;
                _spriteSheetRows = _spriteSheetImage.height / _spriteHeight;
                _numFrames = _spriteSheetColumns * _spriteSheetRows;
            }

            if (target && source)
            {
                updateCanvas();
            }
        }

        /**
         * re-draw the frame to the canvas
         */
        protected function updateCanvas():void
        {
            if (_spriteSheetImage)
            {
                _tempRect.x = (_currentFrame % _spriteSheetColumns) * _spriteWidth;
                _tempRect.y = Math.floor(_currentFrame / _spriteSheetColumns) * _spriteHeight;
                _tempRect.width = _spriteWidth;
                _tempRect.height = _spriteHeight;
                _canvas.copyPixels( _spriteSheetImage, _tempRect, ORIGIN, true );
            }
        }
    }
}
